from typing import Any, Callable, Literal, TypeVar, get_origin

from pydantic import TypeAdapter

T = TypeVar("T")


def parse_as(
    type_: type[T],
    data: Any,
    mode: Literal["python", "json", "strings"] = "python",
) -> T:
    """Parse a given data structure as a Pydantic model via `TypeAdapter`.

    Read more about `TypeAdapter` [here](https://docs.pydantic.dev/latest/concepts/type_adapter/).

    Args:
        type_: The type to parse the data as.
        data: The data to be parsed.
        mode: The mode to use for parsing, either `python`, `json`, or `strings`.
            Defaults to `python`, where `data` should be a Python object (e.g. `dict`).

    Returns:
        The parsed `data` as the given `type_`.

    Example:
        ```python
        from gh.types import GitHubIssue
        from gh.utils import parse_as

        issue = parse_as(GitHubIssue, {"title": "Test Issue"})
        print(issue.title)
        # => "Test Issue"
        ```
    """
    adapter = TypeAdapter(type_)

    if get_origin(type_) is list and isinstance(data, dict):
        data = next(iter(data.values()))

    parser: Callable[[Any], T] = getattr(adapter, f"validate_{mode}")

    return parser(data)



from gh_util.functions import fetch_github_issue


async def main():
    issue = await fetch_github_issue("prefecthq", "marvin", 723, include_comments=True)

    print(
        f"[{issue.number} {issue.title}]({issue.url}) was created by {issue.user.login} on {issue.created_at}."
    )

    print(issue.body)

    print()

    for comment in issue.user_comments:
        print(f"[{comment.user.login}]({comment.user.url}) said:")
        print(comment.body)
        print()


if __name__ == "__main__":
    import asyncio

    asyncio.run(main())



from datetime import datetime

from pydantic import BaseModel, ConfigDict, Field, HttpUrl


class GitHubUser(BaseModel):
    model_config = ConfigDict(extra="allow")

    login: str
    url: HttpUrl


class GitHubLabel(BaseModel):
    model_config = ConfigDict(extra="allow")

    id: int
    name: str
    color: str
    description: str | None = None

    def __hash__(self) -> int:
        return hash(self.name)


class GitHubRepo(BaseModel):
    model_config = ConfigDict(extra="allow")

    name: str
    owner: GitHubUser


class GitHubComment(BaseModel):
    model_config = ConfigDict(extra="allow")

    url: HttpUrl
    user: GitHubUser
    body: str
    created_at: datetime
    updated_at: datetime


class GitHubIssue(BaseModel):
    model_config = ConfigDict(extra="allow")

    title: str
    url: HttpUrl
    number: int
    user: GitHubUser
    body: str | None = None

    user_comments: list[GitHubComment] = Field(default_factory=list)

    comments_url: HttpUrl | None = None


class GitHubIssueEvent(BaseModel):
    model_config = ConfigDict(extra="allow")

    action: str
    issue: GitHubIssue
    repository: GitHubRepo



from gh_util.functions import fetch_repo_labels


async def main():
    labels = await fetch_repo_labels("python", "cpython")
    print(labels)


if __name__ == "__main__":
    import asyncio

    asyncio.run(main())



from gh_util.client import GHClient


async def main():
    async with GHClient() as client:
        response = await client.get("/user")
        print(response.json())


if __name__ == "__main__":
    import asyncio

    asyncio.run(main())



import httpx

import gh_util


class GHClient(httpx.AsyncClient):
    """A wrapper around httpx.AsyncClient that adds GitHub authentication."""

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        if gh_util.settings.token:
            self.headers[
                "Authorization"
            ] = f"token {gh_util.settings.token.get_secret_value()}"
        self.headers["Accept"] = "application/vnd.github.v3+json"

    async def request(self, method, url, *args, **kwargs) -> httpx.Response:
        """Allow passing in a relative URL."""
        url = str(url)
        if url.startswith("/"):
            url = f"{gh_util.settings.base_url}{url}"

        response = await super().request(method, url, *args, **kwargs)
        response.raise_for_status()
        return response



from typing import Set

from gh_util.client import GHClient
from gh_util.types import GitHubComment, GitHubIssue, GitHubLabel
from gh_util.utils import parse_as


async def fetch_github_issue(
    owner: str,
    repo: str,
    issue_number: int,
    include_comments: bool = False,
) -> GitHubIssue:
    async with GHClient() as client:
        response = await client.get(f"/repos/{owner}/{repo}/issues/{issue_number}")
        issue = GitHubIssue.model_validate(response.json())

        if include_comments:
            response = await client.get(issue.comments_url)
            issue.user_comments = parse_as(list[GitHubComment], response.json())

        return issue


async def fetch_repo_labels(owner: str, repo: str) -> Set[GitHubLabel]:
    async with GHClient() as client:
        response = await client.get(f"/repos/{owner}/{repo}/labels")
        response.raise_for_status()
        return parse_as(set[GitHubLabel], response.json())


async def add_labels_to_issue(
    owner: str, repo: str, issue_number: int, new_labels: Set[str]
) -> None:
    async with GHClient() as client:
        current_labels_response = await client.get(
            f"/repos/{owner}/{repo}/issues/{issue_number}/labels"
        )

        if labels_to_add := new_labels - {
            label["name"] for label in current_labels_response.json()
        }:
            await client.post(
                f"/repos/{owner}/{repo}/issues/{issue_number}/labels",
                json=list(labels_to_add),
            )



from pydantic import SecretStr
from pydantic_settings import BaseSettings, SettingsConfigDict


class Settings(BaseSettings):
    model_config = SettingsConfigDict(
        env_prefix="GH_UTIL_",
        env_file=".env",
        extra="allow",
        validate_assignment=True,
    )

    token: SecretStr | None = None

    base_url: str = "https://api.github.com"


settings = Settings()
